from enum import Enum, auto


class StrEnum(str, Enum):
    def __new__(cls, value, *args, **kwargs):
        if not isinstance(value, (str, auto)):
            raise TypeError(
                f"Values of StrEnums must be strings: {value!r} is a {type(value)}"
            )
        return super().__new__(cls, value, *args, **kwargs)

    def __str__(self):
        return str(self.value)

    def _generate_next_value_(name, *_):
        return name.lower()


class DatePart(StrEnum):
    """Valid parts to extract from date/time/timestamp arrays.

    See [`date_part`][arro3.compute.date_part].
    """

    Quarter = auto()
    """Quarter of the year, in range `1..=4`"""

    Year = auto()
    """Calendar year"""

    Month = auto()
    """Month in the year, in range `1..=12`"""

    Week = auto()
    """ISO week of the year, in range `1..=53`"""

    Day = auto()
    """Day of the month, in range `1..=31`"""

    DayOfWeekSunday0 = auto()
    """Day of the week, in range `0..=6`, where Sunday is `0`"""

    DayOfWeekMonday0 = auto()
    """Day of the week, in range `0..=6`, where Monday is `0`"""

    DayOfYear = auto()
    """Day of year, in range `1..=366`"""

    Hour = auto()
    """Hour of the day, in range `0..=23`"""

    Minute = auto()
    """Minute of the hour, in range `0..=59`"""

    Second = auto()
    """Second of the minute, in range `0..=59`"""

    Millisecond = auto()
    """Millisecond of the second"""

    Microsecond = auto()
    """Microsecond of the second"""

    Nanosecond = auto()
    """Nanosecond of the second"""



from typing import overload

from arro3.core import Array, ArrayReader
from arro3.core.types import ArrayInput, ArrowStreamExportable

@overload
def filter(
    values: ArrayInput,
    predicate: ArrayInput,
) -> Array: ...
@overload
def filter(
    values: ArrowStreamExportable,
    predicate: ArrowStreamExportable,
) -> ArrayReader: ...
def filter(
    values: ArrayInput | ArrowStreamExportable,
    predicate: ArrayInput | ArrowStreamExportable,
) -> Array | ArrayReader:
    """
    Returns a filtered `values` array where the corresponding elements of
    `predicate` are `true`.

    If `input` is an Array, an `Array` will be returned. If `input` is a `ChunkedArray`
    or `ArrayReader`, an `ArrayReader` will be returned.
    """



from __future__ import annotations

from typing import Literal

DatePartT = Literal[
    "quarter",
    "year",
    "month",
    "week",
    "day",
    "dayofweeksunday0",
    "dayofweekmonday0",
    "dayofyear",
    "hour",
    "minute",
    "second",
    "millisecond",
    "microsecond",
    "nanosecond",
]
"""
Acceptable strings to be passed into the `part` parameter for
[`date_part`][arro3.compute.date_part].
"""



from arro3.core import Scalar
from arro3.core.types import ArrayInput, ArrowStreamExportable

def max(input: ArrayInput | ArrowStreamExportable) -> Scalar:
    """
    Returns the max of values in the array.
    """

def min(input: ArrayInput | ArrowStreamExportable) -> Scalar:
    """
    Returns the min of values in the array.
    """

def sum(input: ArrayInput | ArrowStreamExportable) -> Scalar:
    """
    Returns the sum of values in the array.
    """



from typing import overload

from arro3.core import Array, ArrayReader
from arro3.core.types import ArrayInput, ArrowStreamExportable

@overload
def dictionary_encode(array: ArrayInput) -> Array: ...
@overload
def dictionary_encode(array: ArrowStreamExportable) -> ArrayReader: ...
def dictionary_encode(
    array: ArrayInput | ArrowStreamExportable,
) -> Array | ArrayReader:
    """
    Dictionary-encode array.

    Return a dictionary-encoded version of the input array. This function does nothing if the input is already a dictionary array.

    Note: for stream input, each output array will not necessarily have the same dictionary.

    Args:
        array: Argument to compute function.

    Returns:
        The dictionary-encoded array.
    """



from arro3.compute._aggregate import max as max
from arro3.compute._aggregate import min as min
from arro3.compute._aggregate import sum as sum
from arro3.compute._arith import add as add
from arro3.compute._arith import add_wrapping as add_wrapping
from arro3.compute._arith import div as div
from arro3.compute._arith import mul as mul
from arro3.compute._arith import mul_wrapping as mul_wrapping
from arro3.compute._arith import neg as neg
from arro3.compute._arith import neg_wrapping as neg_wrapping
from arro3.compute._arith import rem as rem
from arro3.compute._arith import sub as sub
from arro3.compute._arith import sub_wrapping as sub_wrapping
from arro3.compute._boolean import is_not_null as is_not_null
from arro3.compute._boolean import is_null as is_null
from arro3.compute._cast import can_cast_types as can_cast_types
from arro3.compute._cast import cast as cast
from arro3.compute._dictionary import dictionary_encode as dictionary_encode
from arro3.compute._filter import filter as filter
from arro3.compute._take import take as take
from arro3.compute._temporal import date_part as date_part



from typing import overload

from arro3.core import Array, ArrayReader
from arro3.core.types import (
    ArrayInput,
    ArrowSchemaExportable,
    ArrowStreamExportable,
)

@overload
def cast(
    input: ArrayInput,
    to_type: ArrowSchemaExportable,
) -> Array: ...
@overload
def cast(
    input: ArrowStreamExportable,
    to_type: ArrowSchemaExportable,
) -> ArrayReader: ...
def cast(
    input: ArrayInput | ArrowStreamExportable,
    to_type: ArrowSchemaExportable,
) -> Array | ArrayReader:
    """
    Cast `input` to the provided data type and return a new Array with type `to_type`, if possible.

    If `input` is an Array, an `Array` will be returned. If `input` is a `ChunkedArray` or `ArrayReader`, an `ArrayReader` will be returned.

    Args:
        input: Input data to cast.
        to_type: The target data type to cast to. You may pass in a `Field` here if you wish to include Arrow extension metadata on the output array.

    Returns:
        The casted Arrow data.
    """

def can_cast_types(
    from_type: ArrowSchemaExportable, to_type: ArrowSchemaExportable
) -> bool:
    """Return true if a value of type `from_type` can be cast into a value of `to_type`.

    Args:
        from_type: Source type
        to_type: Destination type

    Returns:
        True if can be casted.
    """
